﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading;
using System.Threading.Tasks;
using System.Windows.Forms;
using Microsoft.Extensions.Logging.Abstractions;
using RetailForce.Common.Validation;
using RetailForce.Fiscalisation;
using RetailForce.Fiscalisation.Cloud;
using RetailForce.Fiscalisation.Configuration;
using RetailForce.Fiscalisation.Implementation.NoFiscalisation;
using RetailForce.Fiscalisation.Model;

namespace ArchiveOnline
{
    public partial class SampleForm : Form
    {
        // This is the data for the client
        private string clientVatId = "YOURUID";
        private string clientTaxId = "YOUTAXNR";
        private string clientStoreNumber = "CASHTEST01";
        private string clientTerminalNumber = "T001";
        private string cloudApiKey = "Fill in key";
        private string cloudApiSecret = "Fill your secret";

        #region Variable declaration
        private TrustedFiscalModule _fiscalModule;
        private TrustedFiscalModuleNoFiscalisation _fiscalModuleNoFiscalisation;
        private FileConfigurationProvider _configurationProvider;
        #endregion

        public SampleForm()
        {
            InitializeComponent();
        }

        private void SampleForm_Load(object sender, EventArgs e)
        {
            // create configuration provider (in this case file configuration, but it is also possible to store the configuration anywhere)
            // you only have to implement a configuration provider if the configuration should be stored to another location
            _configurationProvider = new FileConfigurationProvider(Path.Combine(Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location), "fiscalclient.config"), NullLogger.Instance);

            // class to manage fiscal module -> you can check of the given client is already installed
            FiscalModuleManagement management = new FiscalModuleManagement(NullLogger.Instance, _configurationProvider);

            // get all clients which are currently stored in configuration
            List<FiscalClient> clients = management.GetClients();

            // check if the client you need is already stored in the configuration
            Guid? clientId = null;
            if (clients == null || clients.Count == 0 || !clients.Any(a => a.StoreNumber == clientStoreNumber && a.TerminalNumber == clientTerminalNumber && a.CompanyIdentification.Any(x => x.Type == CompanyIdentification.IdentificationType.VatNumber && x.Identification == clientVatId)))
            {
                // no client exists
                // Possibility 1: CreateClient With configuration from cloud
                // clientId = management.CreateClientByCloud(new CompanyIdentification() {Identification = clientVatId, Type = CompanyIdentification.IdentificationType.VatNumber}, clientStoreNumber, clientTerminalNumber, cloudApiKey, cloudApiSecret);
                // Possibility 2: CreateClient with your own configuration (you have to know the id of the client in the cloud, or to enter)
                clientId = Guid.Parse("56458aa1-64c4-48ba-a1ae-3ae88be453e1");
                FiscalClient fiscalClient = new FiscalClient()
                {
                    UniqueClientId = clientId.Value,
                    CompanyIdentification = new CompanyIdentification[]
                    {
                        new CompanyIdentification()
                        {
                            Identification = clientVatId, Type = CompanyIdentification.IdentificationType.VatNumber
                        },
                        new CompanyIdentification()
                        {
                            Identification = clientTaxId, Type = CompanyIdentification.IdentificationType.TaxNumber
                        }
                    },
                    CompanyAddress = new Address()
                    {
                        CountryCode = "DEU",
                        City = "Teststadt",
                        PostalCode = "44000",
                        Street = "Teststrasse",
                        StreetNumber = "1"
                    },
                    FiscalCountry = FiscalCountry.Germany,
                    CashRegister = new CashRegister()
                    {
                        Brand = "x",
                        CurrencyIsoCode = "EUR",
                        Id = "x",
                        Modelname = "x",
                        Software = new Software()
                        {
                            Brand = "PCCASH",
                            Version = "1.0"
                        }
                    },
                    CompanyName = "TestCompany",
                    DistributerId = Guid.Parse("e8084763-cf8f-4367-a372-9b4d4323d65a"), // Distributor Id -> replace with 
                    LicenseConsumerId = Guid.Parse("4ced945e-7e78-49f6-8bc3-aa3b774c8bc7"), // Testorganisation Archivierung Id 
                    FiscalModulImplementationConfiguration = new ClientConfiguration(),
                    StoreNumber = clientStoreNumber,
                    TestClient = true,
                    StoreName = "TestStore",
                    StoreAddress = new Address()
                    {
                        CountryCode = "DEU",
                        City = "Teststadt",
                        PostalCode = "44000",
                        Street = "Teststrasse",
                        StreetNumber = "1"
                    },
                    TerminalNumber = clientTerminalNumber
                };
                List<ValidationError> errorList = fiscalClient.Validate();

                management.CreateClient(fiscalClient);
            }
            else
                clientId = clients.First(a => a.StoreNumber == clientStoreNumber && a.TerminalNumber == clientTerminalNumber && a.CompanyIdentification.Any(x => x.Type == CompanyIdentification.IdentificationType.VatNumber && x.Identification == clientVatId)).UniqueClientId;

            if (!clientId.HasValue)
                throw new InvalidOperationException("Client not found.");

            // now create fiscalmodule
            CloudService cloudService = new CloudService(NullLogger.Instance, Path.GetDirectoryName(System.Reflection.Assembly.GetEntryAssembly().Location));
            FiscalModulCreator creator = new FiscalModulCreator(_configurationProvider, NullLogger.Instance, cloudService);
            _fiscalModule = creator.CreateFiscalModuleForClient(clientId.Value, cloudApiKey: cloudApiKey, cloudApiSecret: cloudApiSecret);
            if (_fiscalModule == null)
                throw new InvalidOperationException("Fiscalmodule cannot be created.");

            // and german implementation for archive is in no fiscalisation module (here are the methods for archiving)
            _fiscalModuleNoFiscalisation = (TrustedFiscalModuleNoFiscalisation)_fiscalModule.ModulImplementation;
        }

        private void btnUpload_Click(object sender, EventArgs e)
        {
            List<UploadInfo> uploadInfo = new List<UploadInfo>();

            if (!string.IsNullOrEmpty(txtDSFINVKFolder.Text) && Directory.Exists(txtDSFINVKFolder.Text))
            {
                uploadInfo.Add(new UploadInfo()
                {
                    Type = 1,
                    Entity = txtDSFINVKFolder.Text
                });
            }

            if (!string.IsNullOrEmpty(txtBackupFolder.Text) && Directory.Exists(txtBackupFolder.Text))
            {
                uploadInfo.Add(new UploadInfo()
                {
                    Type = 2,
                    Entity = txtBackupFolder.Text,
                    Container = "backup1.zip",
                    Recursive = true
                });
            }

            if (uploadInfo.Count > 0)
            {
                _fiscalModuleNoFiscalisation.CloudArchiveUploadGermany(uploadInfo);
                MessageBox.Show("Upload successful.");
            }
        }

        private void btnDownload_Click(object sender, EventArgs e)
        {
            if (!string.IsNullOrEmpty(txtDownloadFolder.Text) && Directory.Exists(txtDownloadFolder.Text))
            {
                // ensure to have "/" at the end of the path
                //if (!txtDownloadFolder.Text.EndsWith(@"\"))
                //    txtDownloadFolder.Text += @"\";

                _fiscalModuleNoFiscalisation.CloudArchiveDownloadGermany(DateTime.Now.Date.AddDays(-1), DateTime.Now.Date, txtDownloadFolder.Text);

                MessageBox.Show("Download successful.");
            }
        }
    }
}
